@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  *
- $Id: battstat_applet.c,v 1.114 2005/08/27 18:13:24 davyd Exp $
+ $Id: battstat_applet.c,v 1.122 2006/01/30 10:48:14 davyd Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -25,6 +25,8 @@
 
 #include <stdio.h>
 
+#include <sys/types.h>
+#include <sys/wait.h>
 #include <err.h>
 #include <stdlib.h>
 #include <string.h>
@@ -384,42 +386,37 @@ static gboolean
 battery_full_notify (GtkWidget *applet)
 {
 #ifdef HAVE_LIBNOTIFY
-	static NotifyIcon *icon = NULL;
-	NotifyHints *hints;
-	GtkRequisition size;
-	int x, y;
+	GError *error = NULL;
+	GdkPixbuf *icon;
+	gboolean result;
 	
 	if (!notify_is_initted () && !notify_init (_("Battery Monitor")))
 		return FALSE;
 
-	/* FIXME - this is an issue in something... */
-	/* if (!icon)
-		icon = notify_icon_new_from_uri ("gnome-dev-battery"); */
-
-	/* get the position of the applet on the panel */
-	gdk_window_get_origin (applet->window, &x, &y);
-	gtk_widget_size_request (applet, &size);
-	x += size.width / 2;
-	y += size.height;
-
-	hints = notify_hints_new ();
-	notify_hints_set_int (hints, "x", x);
-	notify_hints_set_int (hints, "y", y);
+  	icon = gtk_icon_theme_load_icon (
+			gtk_icon_theme_get_default (),
+			"gnome-dev-battery",
+			48,
+			GTK_ICON_LOOKUP_USE_BUILTIN,
+			NULL);
 	
-	/* send the notification */
-	if (!notify_send_notification (NULL,
-				"device",
-				NOTIFY_URGENCY_NORMAL,
-				_("Your battery is now fully recharged"),
-				NULL,		/* body text */
-				icon,		/* icon */
-				TRUE, 0,	/* expiry, server default */
-				hints,		/* hints */
-				NULL,		/* no user_data */
-				0))		/* no actions */
-		return FALSE;
+	NotifyNotification *n = notify_notification_new (_("Your battery is now fully recharged"), "", /* "gnome-dev-battery" */ NULL, applet);
 
-	return TRUE;
+	/* XXX: it would be nice to pass this as a named icon */
+	notify_notification_set_icon_from_pixbuf (n, icon);
+	gdk_pixbuf_unref (icon);
+
+	result = notify_notification_show (n, &error);
+	
+	if (error)
+	{
+	   g_warning (error->message);
+	   g_error_free (error);
+	}
+
+	g_object_unref (G_OBJECT (n));
+
+	return result;
 #else
 	return FALSE;
 #endif
@@ -487,11 +484,37 @@ battery_low_dialog_destroy( ProgressData *battstat )
   battstat->battery_low_label = NULL;
 }
 
+/* Determine if suspend is unsupported.  For the time being this involves
+ * distribution-specific magic :(
+ */
+/* #define HAVE_PMI */
+static gboolean
+is_suspend_unavailable( void )
+{
+#ifdef HAVE_PMI
+  int status;
+
+  status = system( "pmi query suspend" );
+
+  /* -1 - fail (pmi unavailable?).     return 'false' since we don't know.
+   * 0  - success (can suspend).       return 'false' since not unavailable.
+   * 1  - success (cannot suspend).    return 'true' since unavailable.
+   */
+  if( WEXITSTATUS( status ) == 1 )
+    return TRUE;
+  else
+    return FALSE;
+#else
+  return FALSE; /* return 'false' since we don't know. */
+#endif
+}
+
 /* Update the text label in the battery low dialog.
  */
 static void
 battery_low_update_text( ProgressData *battstat, BatteryStatus *info )
 {
+  const char *suggest;
   gchar *remaining, *new_label;
   GtkRequisition size;
 
@@ -519,18 +542,27 @@ battery_low_update_text( ProgressData *battstat, BatteryStatus *info )
                                  info->minutes ),
                                info->minutes,info->percent );
 
-  new_label = g_strdup_printf (
+  if( is_suspend_unavailable() )
+    /* TRANSLATORS: this is a list, it is left as a single string
+     * to allow you to make it appear like a list would in your
+     * locale.  This is if the laptop does not support suspend. */
+    suggest = _("To avoid losing your work:\n"
+                " \xE2\x80\xA2 plug your laptop into external power, or\n"
+                " \xE2\x80\xA2 save open documents and shut your laptop down."
+               );
+  else
+    /* TRANSLATORS: this is a list, it is left as a single string
+     * to allow you to make it appear like a list would in your
+     * locale.  This is if the laptop supports suspend. */
+    suggest = _("To avoid losing your work:\n"
+                " \xE2\x80\xA2 suspend your laptop to save power,\n"
+                " \xE2\x80\xA2 plug your laptop into external power, or\n"
+                " \xE2\x80\xA2 save open documents and shut your laptop down."
+               );
+
+  new_label = g_strdup_printf(
 		"<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s\n\n%s",
-		_("Your battery is running low"),
-		remaining,
-		/* TRANSLATORS: this is a list, it is left as a single string
-		 * to allow you to make it appear like a list would in your
-		 * locale. */
-		_("To avoid losing your work:\n"
-		  " \xE2\x80\xA2 suspend your laptop to save power,\n"
-		  " \xE2\x80\xA2 plug your laptop into external power, or\n"
-		  " \xE2\x80\xA2 save open documents and shut your laptop down."
-		  ));
+		_("Your battery is running low"), remaining, suggest );
 
   gtk_label_set_markup( battstat->battery_low_label, new_label );
   g_free( remaining );
@@ -557,6 +589,8 @@ battery_low_dialog( ProgressData *battery, BatteryStatus *info )
 		 GTK_STOCK_OK,
 		 GTK_RESPONSE_ACCEPT,
 		 NULL);
+  gtk_dialog_set_default_response( GTK_DIALOG (battery->battery_low_dialog),
+                                   GTK_RESPONSE_ACCEPT );
 
   g_signal_connect_swapped( GTK_OBJECT (battery->battery_low_dialog),
                             "response",
@@ -595,6 +629,8 @@ battery_low_dialog( ProgressData *battery, BatteryStatus *info )
 
   battery_low_update_text( battery, info );
 
+  gtk_window_set_position (GTK_WINDOW (battery->battery_low_dialog),
+		  GTK_WIN_POS_CENTER);
   gtk_widget_show_all (battery->battery_low_dialog);
 }
 
@@ -1016,6 +1052,8 @@ check_for_updates( gpointer data )
        battstat->last_batt_life != info.percent) ||
       (battstat->showtext == APPLET_SHOW_TIME &&
        battstat->last_minutes != info.minutes) ||
+       battstat->last_acline_status != info.on_ac_power ||
+       battstat->last_present != info.present ||
        battstat->refresh_label ) /* set by properties dialog */
   {
     /* Update the label */
@@ -1029,6 +1067,7 @@ check_for_updates( gpointer data )
   battstat->last_batt_life = info.percent;
   battstat->last_minutes = info.minutes;
   battstat->last_acline_status = info.on_ac_power;
+  battstat->last_present = info.present;
 
   return TRUE;
 }
@@ -1121,8 +1160,11 @@ about_cb( BonoboUIComponent *uic, ProgressData *battstat, const char *verb )
     NULL
   };
 
-  if( power_management_using_hal() )
-    authors[4] = "Ryan Lortie <desrt@desrt.ca> \xe2\x98\x86";
+  char *comments = g_strdup_printf ("%s\n\n%s",
+		  _("This utility shows the status of your laptop battery."),
+		  power_management_using_hal () ?
+		  	/* true */ _("HAL backend enabled.") :
+			/* false */ _("Legacy (non-HAL) backend enabled."));
 
   gtk_show_about_dialog( NULL,
     "name",                _("Battery Charge Monitor"), 
@@ -1130,15 +1172,32 @@ about_cb( BonoboUIComponent *uic, ProgressData *battstat, const char *verb )
     "copyright",           "\xC2\xA9 2000 The Gnulix Society, "
                            "\xC2\xA9 2002-2005 Free Software Foundation and "
                            "others",
-    "comments",            _("This utility shows the status of your laptop "
-                             "battery."),
+    "comments",            comments,
     "authors",             authors,
     "documenters",         documenters,
     "translator-credits",  _("translator-credits"),
     "logo-icon-name",      "gnome-dev-battery",
     NULL );
+
+  g_free (comments);
 }
 
+/* Rotate text on side panels.  Called on initial startup and when the
+ * orientation changes (ie: the panel we were on moved or we moved to
+ * another panel).
+ */
+static void
+setup_text_orientation( ProgressData *battstat )
+{
+  if( battstat->orienttype == PANEL_APPLET_ORIENT_RIGHT )
+    gtk_label_set_angle( GTK_LABEL( battstat->percent ), 90 );
+  else if( battstat->orienttype == PANEL_APPLET_ORIENT_LEFT )
+    gtk_label_set_angle( GTK_LABEL( battstat->percent ), 270 );
+  else
+    gtk_label_set_angle( GTK_LABEL( battstat->percent ), 0 );
+}
+
+
 /* This signal is delivered by the panel when the orientation of the applet
    has changed.  This is either because the applet has just been created,
    has just been moved to a new panel or the panel that the applet was on
@@ -1159,6 +1218,7 @@ change_orient (PanelApplet       *applet,
     /* The applet changing orientation very likely involves the layout
        being changed to better fit the new shape of the panel.
     */
+    setup_text_orientation( battstat );
     reconfigure_layout( battstat );
   }
 }
@@ -1184,52 +1244,6 @@ size_allocate( PanelApplet *applet, GtkAllocation *allocation,
   reconfigure_layout( battstat );
 }
 
-/* Some vaguely magic/evil code to handle coloured or transparent panels.
-   The panel currently doesn't emit these signals often enough, so sometimes
-   we don't get properly updated.
-*/
-static void
-change_background (PanelApplet *a,
-		   PanelAppletBackgroundType type,
-		   GdkColor *color,
-		   GdkPixmap *pixmap,
-		   ProgressData *battstat)
-{
-	/* taken from the Trash Applet */
-	GtkRcStyle *rc_style;
-	GtkStyle *style;
-
-	/* reset style */
-	gtk_widget_set_style (GTK_WIDGET (battstat->applet), NULL);
-	rc_style = gtk_rc_style_new ();
-	gtk_widget_modify_style (GTK_WIDGET (battstat->applet), rc_style);
-	gtk_rc_style_unref (rc_style);
-
-	switch (type) {
-		case PANEL_COLOR_BACKGROUND:
-			gtk_widget_modify_bg (GTK_WIDGET (battstat->applet),
-					GTK_STATE_NORMAL, color);
-			break;
-
-		case PANEL_PIXMAP_BACKGROUND:
-			style = gtk_style_copy
-				(GTK_WIDGET (battstat->applet)->style);
-			if (style->bg_pixmap[GTK_STATE_NORMAL])
-				g_object_unref
-					(style->bg_pixmap[GTK_STATE_NORMAL]);
-			style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref
-				(pixmap);
-			gtk_widget_set_style (GTK_WIDGET (battstat->applet),
-					style);
-			g_object_unref (style);
-			break;
-
-		case PANEL_NO_BACKGROUND:
-		default:
-			break;
-	}
-}
-
 /* Get our settings out of gconf.
  */
 static void
@@ -1460,6 +1474,10 @@ create_layout(ProgressData *battstat)
 {
   if (DEBUG) g_print("create_layout()\n");
 
+  /* Have our background automatically painted. */
+  panel_applet_set_background_widget( PANEL_APPLET( battstat->applet ),
+                                      GTK_WIDGET( battstat->applet ) );
+
   /* Allocate the four widgets that we need. */
   battstat->table = gtk_table_new( 3, 3, FALSE );
   battstat->percent = gtk_label_new( "" );
@@ -1515,11 +1533,6 @@ create_layout(ProgressData *battstat)
    		    G_CALLBACK (size_allocate),
 		    battstat);
 
-  g_signal_connect (battstat->applet,
-		    "change_background",
-		    G_CALLBACK (change_background),
-		    battstat);
-
   return FALSE;
 }
 
@@ -1568,6 +1581,7 @@ battstat_applet_fill (PanelApplet *applet)
 
   load_preferences (battstat);
   create_layout (battstat);
+  setup_text_orientation( battstat );
 
   battstat->pixtimer = gtk_timeout_add (1000, check_for_updates, battstat);
 
