@@ -1,7 +1,7 @@
 #ifndef __GWEATHER_APPLET_H_
 #define __GWEATHER_APPLET_H_
 
-/* $Id: gweather-applet.h,v 1.8 2003/07/13 23:17:52 kevinv Exp $ */
+/* $Id: gweather-applet.h,v 1.10 2006/01/12 05:48:25 philipl Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
@@ -13,13 +13,13 @@
  *
  */
 
-#include "weather.h"
 #include "gweather.h"
 
 G_BEGIN_DECLS
 
 extern void gweather_applet_create(GWeatherApplet *gw_applet);
 extern gint timeout_cb (gpointer data);
+extern gint suncalc_timeout_cb (gpointer data);
 extern void gweather_update (GWeatherApplet *applet);
 extern void gweather_info_load (const gchar *path, GWeatherApplet *applet);
 extern void gweather_info_save (const gchar *path, GWeatherApplet *applet);
